package main

import (
	"bufio"
	"fmt"
	"os"

	"time"

	"code.cloudfoundry.org/lager"
	"github.com/pivotal-cf-experimental/replication-canary/alert"
	"github.com/pivotal-cf-experimental/replication-canary/canary"
	"github.com/pivotal-cf-experimental/replication-canary/config"
	"github.com/pivotal-cf-experimental/replication-canary/database"
	"github.com/pivotal-cf-experimental/replication-canary/galera"
)

func main() {
	appConfig, err := config.NewConfig(os.Args)

	logger := appConfig.Logger

	if err != nil {
		logger.Fatal("Failed to read config", err, lager.Data{
			"config": sanitizeConfig(appConfig),
		})
	}

	err = appConfig.Validate()
	if err != nil {
		logger.Fatal("Failed to validate config", err, lager.Data{
			"config": sanitizeConfig(appConfig),
		})
	}

	scanner := bufio.NewScanner(os.Stdin)
	fmt.Printf("Enter timestamp: ")
	scanner.Scan()
	input := scanner.Text()
	if err = scanner.Err(); err != nil {
		logger.Fatal("Error reading input", err)
	}

	timestamp, err := time.Parse(time.RFC3339, input)
	if err != nil {
		logger.Fatal("Parsing timestamp failed", err)
	}

	connFactory := database.NewConnectionFactoryFromConfig(
		appConfig,
		nil,
		logger,
	)
	conns, err := connFactory.Conns()
	if err != nil {
		logger.Fatal("error creating connections", err)
	}

	databaseSession := make(map[string]string)
	databaseSession["wsrep_sync_wait"] = "1"

	client := database.NewClient(databaseSession, logger)
	healthchecker := &galera.Client{
		Logger: logger,
	}
	bird := canary.NewCanary(client, healthchecker, 0*time.Second, logger)

	replicationSuccess, err := bird.Read(conns, timestamp)
	if err != nil {
		logger.Fatal("Checking replication status failed", err)
	}

	// this could also be nil
	aggregateAlerter := alert.AggregateAlerter{}

	coalMiner := canary.NewCoalMiner(
		connFactory,
		bird,
		aggregateAlerter,
		logger,
	)

	coalMiner.ParseReplicationHealth(replicationSuccess, timestamp, err)
}

func sanitizeConfig(cfg *config.Config) *config.Config {
	cfg.Canary.Password = "REDACTED"

	return cfg
}
