package main

import (
	"os"

	"time"

	"code.cloudfoundry.org/lager"
	"github.com/pivotal-cf-experimental/replication-canary/canary"
	"github.com/pivotal-cf-experimental/replication-canary/config"
	"github.com/pivotal-cf-experimental/replication-canary/database"
	"github.com/pivotal-cf-experimental/replication-canary/galera"
	"github.com/pivotal-cf-experimental/replication-canary/switchboard"
)

func main() {
	appConfig, err := config.NewConfig(os.Args)

	logger := appConfig.Logger

	if err != nil {
		logger.Fatal("Failed to read config", err, lager.Data{
			"config": sanitizeConfig(appConfig),
		})
	}

	err = appConfig.Validate()
	if err != nil {
		logger.Fatal("Failed to validate config", err, lager.Data{
			"config": sanitizeConfig(appConfig),
		})
	}

	switchboardURL := appConfig.Switchboard.URLs[0]
	skipSSLCertVerify := true
	switchboardClient := switchboard.NewClient(
		switchboardURL,
		appConfig.Switchboard.Username,
		appConfig.Switchboard.Password,
		skipSSLCertVerify,
		logger,
	)

	connFactory := database.NewConnectionFactoryFromConfig(
		appConfig,
		switchboardClient,
		logger,
	)

	databaseSession := make(map[string]string)
	databaseSession["wsrep_sync_wait"] = "1"

	client := database.NewClient(databaseSession, logger)
	healthchecker := &galera.Client{
		Logger: logger,
	}
	bird := canary.NewCanary(client, healthchecker, 0*time.Second, logger)

	writeConn, err := connFactory.WriteConn()
	if err != nil {
		logger.Fatal("Canary setup failed", err)
	}

	err = bird.Setup(writeConn)
	if err != nil {
		logger.Fatal("Canary setup failed", err)
	}

	timestamp := time.Now()
	err = bird.Write(timestamp, writeConn)
	if err != nil {
		logger.Fatal("Writing to database failed", err)
	}

	logger.Info("successfully wrote timestamp", lager.Data{
		"timestamp": timestamp,
	})
}

func sanitizeConfig(cfg *config.Config) *config.Config {
	cfg.Canary.Password = "REDACTED"

	return cfg
}
